<?php

// $Id: fast_gallery.class.php,v 1.24 2009/04/08 14:26:56 rapsli Exp $

/**
 * @file
 * This is a helper class doing most of the database calls.
 *
 * @author Raphael Schr - www.schaerwebdesign.ch
 * @author Nicholas C. Yang - www.nyanginator.com
 */

class FastGallery {
  private static $instance = null;

  /**
   * We are implementing a singleton pattern
   */
  private function __construct() {
  }

  public function getInstance() {
    if (is_null(self :: $instance)) {
      self :: $instance = new self;
    }
    return self :: $instance;
  }

  /**
   * Checks for gallery additions/deletions only. Leaves existing files alone.
   */
  public function rescanGallery() {
    $path = variable_get('fast_gallery_path', '');
    $path = utf8_decode($path);

    // Get current file list hierarchy
    $files = $this->exploreDir($path, TRUE);

    // Delete non-existent files
    $this->deletePics();

    // Update database with new files
    $this->updateDb($files);

    // Check for orphan images/folders and fix it
    $this->checkHierarchy();
  }

  /**
   * Write the image information to the database so it can be processed
   * faster later on.
   *
   * @param ar_files
   *   Array of files, as returned by exploreDir()
   */
  public function updateDb($ar_files) {
    foreach ($ar_files as $key => $value) {

      // Handle files with special characters.
      $value_slash = str_replace('\\', '/', utf8_encode($value));

      // Skip over the custom folder thumbnails.
      $tmp_ar = explode('/', $value_slash);
      if (end($tmp_ar) == 'folder.jpg')
        continue;

      //we don't care about empty arrays
      if (is_array($value) && count($value) < 1) {
        continue;
      }

      // If we found a folder, recurse through it
      if (is_array($value)) {
        $this->updateDb($value);
      } else {
        // Get the image's folder and parent
        $arTmp = explode('/', $value_slash);

        $folder = $this->subpath($arTmp, 1);
        $parent = $this->subpath($arTmp, 2);

        // Get rid of spaces in URL path
        $folder = str_replace(' ', '_', $folder);
        $parent = str_replace(' ', '_', $parent);

        // Write record to the database. Exclude non-image custom thumbnails.
        $tmp_ar = explode('.', $value);
        array_pop($tmp_ar);

        if (end($tmp_ar) != 'screenshot') {
          $dbObject['path'] = $value_slash;
          $dbObject['filetype'] = strtolower(end(explode('.', $value)));
          $dbObject['folder'] = $folder;
          $dbObject['parent'] = $parent;

          // Check for a previous entry in the database
          $sql = "SELECT gid FROM {fast_gallery} WHERE path = '%s' AND folder = '%s'";
          $result = db_query($sql, $value_slash, $folder);
          $row = db_fetch_object($result);

          // Only write a record to the database if it's new
          if ($row == NULL || $row == '') {
            drupal_write_record('fast_gallery', $dbObject);
            $gid = db_last_insert_id('{fast_gallery}', 'gid');
          }
          // Otherwise grab the GID of this record so we can check its EXIF
          // data for any changes
          else {
            $gid = $row->gid;
          }

          // Update EXIF table data; some hosters have problems so this
          // if-statement is necessary
          if (variable_get('fast_gallery_use_exif', 1)) {
            if (function_exists('exif_read_data')) {
              $this->updateExifDb($value_slash, $gid);
            } else {
              drupal_set_message('EXIF data not supported on your host.');
            }
          }
        }
      }
    }
  }

  /**
   * Given a path and index n, removes n number of elements from the end of
   * the path (delimited by '/').
   *
   * @param path_ar
   *   Path.
   * @param index
   *   Number of elements to remove from end of the path.
   */
  public function subpath($path_ar, $index) {
    $path_ar = array_slice($path_ar, 0, - $index);
    return implode('/', $path_ar);
  }

  /**
   * Check for orphan images/folders and inserts appropriate database
   * records to link them into the hierarchy. Necessary for nested folders
   * that are in folders with no other images.
   */
  public function checkHierarchy() {
    // Get the name of the folder that is the gallery root
    $abs_path = variable_get('fast_gallery_path', '');
    $abs_path = utf8_decode($abs_path);
    $path_folder = explode('/', $abs_path);
    array_pop($path_folder);
    $abs_path_folder = str_replace(' ', '_', end($path_folder));

    // Get a list of folders that are currently associated with images
    $result = db_query("SELECT DISTINCT folder FROM {fast_gallery}");

    // Put it all in an array. Don't forget to add the gallery root folder.
    $folders = array ();
    $folders[] = $abs_path_folder;
    while ($row = db_fetch_object($result))
      $folders[] = $row->folder;

    // Now iterate through all images and check to see if their parent is a
    // known folder in the database. If not, then the parent is an orphan and
    // needs to be linked into the hierarchy.
    $result = db_query("SELECT * FROM {fast_gallery}");
    $orphans = array ();
    while ($row = db_fetch_object($result)) {
      // Check if parent is a known folder
      $adopted = 0;
      foreach ($folders as $possible_parent) {
        if ($row->parent == $possible_parent) {
          $adopted = 1;
          break;
        }
      }

      // If the parent is an unknown folder, store it in an array
      if ($adopted == 0) {
        // We only need to add any newly encountered folders once, so check
        // to see that we haven't already added it
        if (!in_array($row->parent, $folders)) {
          // Keep a list of orphans
          $orphans[] = $row;

          // Also, update the current list of known folders
          $folders[] = $row->parent;
        }
      }
    }

    // If we found orphans, then we need to make their parent a known folder
    if (count($orphans) != 0) {
      foreach ($orphans as $file) {
        $path = $file->path;

        $ar_path = explode('/', str_replace(' ', '_', $path));

        $folder = $this->subpath($ar_path, 2);
        $parent = $this->subpath($ar_path, 3);

        // Check for a previous entry in the database
        // if we don't we get double entries... -> bug http://drupal.org/node/368355
        $sql = "SELECT gid FROM {fast_gallery} WHERE path = '%s' AND folder = '%s'";
        $result = db_query($sql, $path, $folder);
        $row = db_fetch_object($result);

        // Only write the record if we're not past the root directory
        if ($folder != NULL && $parent != NULL && $row == NULL) {
          $dbObject['path'] = $path;
          $dbObject['filetype'] = strtolower(end(explode('.', $file->path)));
          $dbObject['folder'] = $folder;
          $dbObject['parent'] = $parent;

          //print_r($dbObject);
          drupal_write_record('fast_gallery', $dbObject);
          $gid = db_last_insert_id('{fast_gallery}', 'gid');

          // Update EXIF table data
          if (variable_get('fast_gallery_use_exif', 1)) {
            if (function_exists('exif_read_data'))
              $this->updateExifDb($file->path, $gid);
            else
              drupal_set_message('EXIF data not supported on your host.');
          }
        }

        // Go to next directory level up
        array_pop($ar_path);

        // Reiterate the process. Check if the parent is a known folder.
        // If not, add it in. By the end, all unknown parents should be
        // known folders.
        while (count($ar_path) > count($path_folder)) {
          if (count($ar_path) >= 4) {
            $folder = $this->subpath($ar_path, 2);
            $parent = $this->subpath($ar_path, 3);

            // Check if the parent is a known folder
            $adopted = 0;
            foreach ($folders as $possible_parent) {
              if ($folder == $possible_parent) {
                $adopted = 1;
                break;
              }
            }

            // If the parent is an unknown folder, write it to the database
            if ($adopted == 0) {
              if (!in_array($folder, $folders)) {
                $dbObject['path'] = $path;
                $dbObject['filetype'] = strtolower(end(explode('.', $file->path)));
                $dbObject['folder'] = $folder;
                $dbObject['parent'] = $parent;
                drupal_write_record('fast_gallery', $dbObject);
                $folders[] = $folder;

                $gid = db_last_insert_id('{fast_gallery}', 'gid');

                // Update EXIF table data
                if (variable_get('fast_gallery_use_exif', 1)) {
                  if (function_exists('exif_read_data')) {
                    $this->updateExifDb($file->path, $gid);
                  } else {
                    drupal_set_message('EXIF data not supported on your host.');
                  }
                }
              }
            }
          }

          // Go to the next directory level up and reiterate
          array_pop($ar_path);
        }
      }
    }
  }

  /**
   * Write the image's EXIF information to the database.
   *
   * @param file
   *   Path of the image file.
   * @param gid
   *   ID of the picture assigned by the database.
   */
  public function updateExifDb($file, $gid) {
    // Get the file extension
    $tmp = explode('.', $file);
    $fileType = strtolower(end($tmp));

    if ($fileType != 'jpg' && $fileType != 'jpeg') {
      // Construct the custom thumbnail path
      $tmp_ar = explode('.', $file);
      array_pop($tmp_ar);
      $tmp = implode('.', $tmp_ar);
      $file = $tmp . '.screenshot.jpg';
    }

    // Grab the EXIF data that we're interested in
    if (file_exists(utf8_decode($file)))
      $exif = exif_read_data(utf8_decode($file), 0, true);
    $param['model'] = $exif['IFD0']['Model'];
    $param['exposureTime'] = $exif['EXIF']['ExposureTime'];
    $param['dateTaken'] = $exif['EXIF']['DateTimeOriginal'];
    $param['ISOSpeedRatings'] = $exif['EXIF']['ISOSpeedRatings'];
    $param['description'] = utf8_encode($exif['IFD0']['ImageDescription']);
    $param['comment'] = utf8_encode($exif['COMPUTED']['UserComment']);
    if (file_exists($file)) { //else will return error on videos
      $param['fileCreated'] = filemtime(utf8_decode($file));
    }
    $param['copyright'] = utf8_encode($exif['COMPUTED']['Copyright']);
    $param['artist'] = utf8_encode($exif['IFD0']['Artist']);

    // Windows XP files data
    $param['xptitle'] = utf8_encode($exif['WINXP']['Title']);
    $param['xpcomments'] = utf8_encode($exif['WINXP']['Comments']);
    $param['xpkeywords'] = utf8_encode($exif['WINXP']['Keywords']);

    // Check for previous entries
    $sql = "SELECT * FROM {fast_gallery_exif} WHERE gid='%s'";
    $result = db_query($sql, $gid);
    $row = db_fetch_object($result);

    // Update entries if they exist
    if ($row != NULL) {
      $existing_record[$row->field] = $row->value;
      while ($row = db_fetch_object($result))
        $existing_record[$row->field] = $row->value;

      if ($existing_record != NULL) {
        foreach ($existing_record as $key => $value) {
          // Make changes to database only if there was a change
          if ($param[$key] != $value) {
            $sql = "UPDATE {fast_gallery_exif} SET value='%s' WHERE gid='%s' AND field='%s'";
            db_query($sql, $param[$key], $gid, $key);
          }
        }
      }
    }
    // Or else write a new record to the database
    else {
      foreach ($param as $key => $value) {
        $dbObject['field'] = $key;
        $dbObject['value'] = $value;
        $dbObject['gid'] = $gid;
        drupal_write_record('fast_gallery_exif', $dbObject);
      }
    }
  }

  /**
   * Find the pics that are not in the filesystem anymore and delete them
   * from the database.
   */
  public function deletePics() {
    // Get unpaged array of all pictures
    $arPics = $this->getAllRecords();

    // Delete the files that don't exist
    foreach ($arPics as $value) {
      if (!file_exists($value->path)) {
        $sql = "DELETE FROM {fast_gallery} WHERE gid = %d";
        db_query($sql, $value->gid);
        $sql = "DELETE FROM {fast_gallery_exif} WHERE gid = %d";
        db_query($sql, $value->gid);
      }
    }
  }

  /**
   * Fetch a random picture from the database.
   *
   * @return
   *   Random image row from the database.
   */
  public function getRandomPicture() {
    $arPics = $this->getAllRecords();
    $randId = array_rand($arPics);
    return $arPics[$randId];
  }

  /**
   * Empty the gallery tables
   */
  public function clearGallery() {
    db_query("TRUNCATE TABLE {fast_gallery}");
    db_query("TRUNCATE TABLE {fast_gallery_exif}");
  }

  /**
   * Get an array of the images in the given folder. Prepend folder elements
   * to the array. Unfortunately for random sorting, large databases will
   * NOT perform well since the RAND() MySQL function is being used.
   *
   * Remember that only JPG files hold EXIF data. If we're sorting by Date
   * Taken or File Created, we have to separately append the non-JPG files
   * to the SQL results.
   *
   * @param folder
   *   Folder from which to get the images and folders
   */
  public function getPicsHierarchy($folder = '') {

    // Get the current sort settings for both images and folders
    $image_direction = variable_get('fast_gallery_direction', 'desc');
    $image_sort_by = variable_get('fast_gallery_sort', 'date');
    $folder_direction = variable_get('fast_gallery_folder_direction', 'desc');
    $folder_sort_by = variable_get('fast_gallery_sort_folder', 'date');
    $num_per_page = variable_get('fast_gallery_nr_per_page', 8);

    // Form the appropriate SQL query for images
    if ($image_sort_by == 'path') {
      $sql_images = "SELECT path, folder, gid FROM {fast_gallery} WHERE folder='%s' " .
      "ORDER by path %s";
    } else
      if ($image_sort_by == 'date') {
        $sql_images = "SELECT path, folder, fg.gid, value FROM {fast_gallery} AS fg, " .
        "{fast_gallery_exif} AS fe WHERE folder='%s' " .
        "AND fg.gid=fe.gid " .
        "AND fe.field='dateTaken' " .
        "ORDER by value %s";
        $sql_images_noexif = "SELECT path, folder, gid, filetype FROM {fast_gallery} " .
        "WHERE folder='%s' " .
        "AND filetype <> 'jpg' " .
        "AND filetype <> 'jpeg' " .
        "AND filetype <> 'flv' " .
        "AND filetype <> 'mov' " .
        "AND filetype <> 'wmv' " .
        "AND filetype <> 'asx' " .
        "AND filetype <> 'swf' " .
        "AND filetype <> 'pdf' " .
        "ORDER by path %s";
      } else
        if ($image_sort_by == 'filec') {
          $sql_images = "SELECT path, folder, fg.gid, value FROM {fast_gallery} AS fg, " .
          "{fast_gallery_exif} AS fe WHERE folder='%s' " .
          "AND fg.gid=fe.gid " .
          "AND fe.field='fileCreated' " .
          "ORDER by value %s";
          $sql_images_noexif = "SELECT path, folder, gid, filetype FROM {fast_gallery} " .
          "WHERE folder='%s' " .
          "AND filetype <> 'jpg' " .
          "AND filetype <> 'jpeg' " .
          "AND filetype <> 'flv' " .
          "AND filetype <> 'mov' " .
          "AND filetype <> 'wmv' " .
          "AND filetype <> 'asx' " .
          "AND filetype <> 'swf' " .
          "AND filetype <> 'pdf' " .
          "ORDER by path %s";
        } else
          if ($image_sort_by == 'rand') {
            $sql_images = "SELECT path, folder, gid FROM {fast_gallery} WHERE folder='%s' " .
            "ORDER by RAND()";
          }

    // Form the appropriate SQL query for folders
    if ($folder_sort_by == 'path') {
      $sql_folders = "SELECT path, fg.folder, gid FROM {fast_gallery} AS fg " .
      "RIGHT JOIN " .
      "(SELECT DISTINCT folder FROM {fast_gallery} WHERE parent='%s') " .
      "AS fldrs " .
      "ON 1 WHERE fg.folder=fldrs.folder GROUP by folder " .
      "ORDER by path %s";
    } else
      if ($folder_sort_by == 'date') {
        $sql_folders = "SELECT path, fg.folder, fg.gid, fe.value FROM {fast_gallery} AS fg, " .
        "{fast_gallery_exif} AS fe " .
        "RIGHT JOIN " .
        "(SELECT DISTINCT folder FROM {fast_gallery} WHERE parent='%s') " .
        "AS fldrs " .
        "ON 1 WHERE fg.folder=fldrs.folder " .
        "AND fg.gid=fe.gid " .
        "AND fe.field='dateTaken' " .
        "GROUP by folder " .
        "ORDER by value %s";
        $sql_folders_noexif = "SELECT path, folder, gid, filetype FROM {fast_gallery} " .
        "WHERE parent='%s' " .
        "AND filetype <> 'jpg' " .
        "AND filetype <> 'jpeg' " .
        "AND filetype <> 'flv' " .
        "AND filetype <> 'mov' " .
        "AND filetype <> 'wmv' " .
        "AND filetype <> 'asx' " .
        "AND filetype <> 'swf' " .
        "AND filetype <> 'pdf' " .
        "GROUP by folder " .
        "ORDER by path %s";
      } else
        if ($folder_sort_by == 'filec') {
          $sql_folders = "SELECT path, fg.folder, fg.gid, fe.value FROM {fast_gallery} AS fg, " .
          "{fast_gallery_exif} AS fe " .
          "RIGHT JOIN " .
          "(SELECT DISTINCT folder FROM {fast_gallery} WHERE parent='%s') " .
          "AS fldrs " .
          "ON 1 WHERE fg.folder=fldrs.folder " .
          "AND fg.gid=fe.gid " .
          "AND fe.field='fileCreated' " .
          "GROUP by folder " .
          "ORDER by value %s";
          $sql_folders_noexif = "SELECT path, folder, gid, filetype FROM {fast_gallery} " .
          "WHERE parent='%s' " .
          "AND filetype <> 'jpg' " .
          "AND filetype <> 'jpeg' " .
          "AND filetype <> 'flv' " .
          "AND filetype <> 'mov' " .
          "AND filetype <> 'wmv' " .
          "AND filetype <> 'asx' " .
          "AND filetype <> 'swf' " .
          "AND filetype <> 'pdf' " .
          "GROUP by folder " .
          "ORDER by path %s";
        } else
          if ($folder_sort_by == 'rand') {
            $sql_folders = "SELECT path, fg.folder, fg.gid, fe.value FROM {fast_gallery} AS fg, " .
            "{fast_gallery_exif} AS fe " .
            "RIGHT JOIN " .
            "(SELECT DISTINCT folder FROM {fast_gallery} WHERE parent='%s') " .
            "AS fldrs " .
            "ON 1 WHERE fg.folder=fldrs.folder " .
            "AND fg.gid=fe.gid " .
            "AND fe.field='fileCreated' " .
            "GROUP by folder " .
            "ORDER by RAND()";
          }

    //fix duplicate temporary table problem.
    // this function is called a couple times if fast gallery is embedded in to a webpage, which cause a table name conflict.
    $table_post_fix = "_" . (rand() % 10000000);
    if (preg_match("/[a-zA-Z]+$/", $folder, $matches)) {
      $table_postfix = $table_post_fix . "_" . $matches[0];
    }
    //end

    // Create temporary tables to make things more readable later on
    db_query_temporary($sql_images, $folder, $image_direction, 'fast_gallery_images');
    db_query_temporary($sql_folders, $folder, $folder_direction, 'fast_gallery_folders');

    // Check if we need to look for and include non-JPG files (no EXIF data)
    $include_imgnoexif = ($image_sort_by == 'date' || $image_sort_by == 'filec');
    $include_foldernoexif = ($folder_sort_by == 'date' || $folder_sort_by == 'filec');

    // For some reason db_query_temporary() returns a boolean instead of a
    // result resource, so we need to re-execute the queries here
    $sql_images = 'SELECT * FROM fast_gallery_images';
    $sql_folders = 'SELECT * FROM fast_gallery_folders';

    // Adjust SQL queries if we need to include non-JPG files with no EXIF data
    if ($include_imgnoexif) {
      db_query_temporary($sql_images_noexif, $folder, $folder_direction, 'fast_gallery_images_noexif');
      $sql_images .= ' UNION SELECT * FROM fast_gallery_images_noexif';
    }
    if ($include_foldernoexif) {
      db_query_temporary($sql_folders_noexif, $folder, $folder_direction, 'fast_gallery_folders_noexif');
      $sql_folders .= ' UNION SELECT * FROM fast_gallery_folders_noexif';
    }

    // Do the requerying and store them in temporary tables for readability
    $result = db_query($sql_images);
    db_query_temporary($sql_images, 'fast_gallery_all_images');
    db_query_temporary($sql_folders, 'fast_gallery_all_folders');

    // We need to specify a count query for pager_query()
    $sql_count = 'SELECT SUM(count) FROM ' .
    '(SELECT COUNT(*) AS count FROM fast_gallery_all_images UNION ' .
    'SELECT COUNT(*) AS count FROM fast_gallery_all_folders) AS tbl';

    // Execute the paged query
    $result = pager_query('SELECT path, folder, gid FROM fast_gallery_all_folders UNION SELECT path, folder, gid FROM fast_gallery_all_images', $num_per_page, 0, $sql_count);

    // Collect the result rows into an array and return it
    $pagedPicBag = array ();
    while ($row = db_fetch_object($result)) {
      if (module_exists('fast_gallery_access')) {
        if (fast_gallery_access_grant_access($row->folder)) {
          $pagedPicBag[] = $row;
        }
      } else {
        $pagedPicBag[] = $row;
      }
    }

    $result = db_query('SELECT path, folder, gid FROM fast_gallery_all_folders UNION SELECT path, folder, gid FROM fast_gallery_all_images');
    $unpagedPicBag = array ();

    while ($row = db_fetch_object($result)) {
      if (module_exists('fast_gallery_access')) {
        if (fast_gallery_access_grant_access($row->folder)) {
          $unpagedPicBag[] = $row;
        }
      } else {
        $unpagedPicBag[] = $row;
      }
      $unpagedPicBag[] = $row;
    }

    return array (
      'paged' => $pagedPicBag,
      'unpaged' => $unpagedPicBag
    );
  }

  /**
   * Get the EXIF data of a picture.
   *
   * @param gid
   *   ID of the picture assigned by the database.
   * @return
   *   EXIF data of picture with specifid gid
   */
  public function getExif($gid) {
    $sql = "SELECT * FROM {fast_gallery_exif} WHERE gid=%d";
    $result = db_query($sql, $gid);
    $out = array ();
    while ($row = db_fetch_object($result))
      $out[$row->field] = $row->value;

    return $out;
  }
  /**
   * Fetch absolutely all the records from the database.
   *
   * @return
   *   Array of all image and folder records in the database
   */
  public function getAllRecords() {
    $sql = 'SELECT gid, path, gid FROM {fast_gallery}';

    $result = db_query($sql);

    $picBag = array ();
    while ($row = db_fetch_object($result))
      $picBag[] = $row;

    return $picBag;
  }

  /**
   * Fetch all the images from the database.
   *
   * @param paged
   *   Specify whether the query should be be paged or not
   * @return
   *   Array of images in the database
   */
  public function getPicsFlat() {
    $num_per_page = variable_get('fast_gallery_nr_per_page', 8);
    $sort_by = variable_get('fast_gallery_sort_flat', 'date');
    $direction = variable_get('fast_gallery_flat_direction', 'desc');

    if ($sort_by == 'path') {
      $sql = "SELECT path, folder, gid FROM {fast_gallery} GROUP by path ORDER by path %s";
    } else
      if ($sort_by == 'date') {
        $sql = "SELECT path, folder, fg.gid, value FROM {fast_gallery} AS fg, " .
        "{fast_gallery_exif} AS fe WHERE " .
        "fg.gid=fe.gid " .
        "AND fe.field='dateTaken' " .
        "GROUP by path ORDER by value %s";
        $sql_noexif = "SELECT path, folder, gid, filetype FROM {fast_gallery} " .
        "WHERE " .
        "filetype <> 'jpg' " .
        "AND filetype <> 'jpeg' " .
        "GROUP by path ORDER by path %s";
      } else
        if ($sort_by == 'filec') {
          $sql = "SELECT path, folder, fg.gid, value FROM {fast_gallery} AS fg, " .
          "{fast_gallery_exif} AS fe WHERE " .
          "fg.gid=fe.gid " .
          "AND fe.field='fileCreated' " .
          "GROUP by path ORDER by value %s";
          $sql_noexif = "SELECT path, folder, gid, filetype FROM {fast_gallery} " .
          "WHERE " .
          "filetype <> 'jpg' " .
          "AND filetype <> 'jpeg' " .
          "GROUP by path ORDER by path %s";
        } else
          if ($sort_by == 'rand') {
            $sql = "SELECT path, folder, gid FROM {fast_gallery} GROUP by path ORDER by RAND()";
          }

    // Create a temporary table
    db_query_temporary($sql, $direction, 'flat');

    // Check if we need to look for and include non-JPG files (no EXIF data)
    $include_noexif = ($sort_by == 'date' || $sort_by == 'filec');

    // db_query_temporary() doesn't return result resource so re-execute SQL
    $sql = 'SELECT * FROM flat';

    // Adjust SQL queries to include non-JPG files with no EXIF data
    if ($include_noexif) {
      db_query_temporary($sql_noexif, $direction, 'flat_noexif');
      $sql .= ' UNION SELECT * FROM flat_noexif';
    }

    // Do the requerying and store them in temporary tables for readability
    db_query_temporary($sql, 'all_flat');

    // We need to specify a count query for pager_query()
    $sql_count = 'SELECT COUNT(*) FROM all_flat';

    // Execute the paged query
    $result = pager_query('SELECT path, folder, gid FROM all_flat', $num_per_page, 0, $sql_count);

    // Collect the results into an array and return it
    $pagedPicBag = array ();
    while ($row = db_fetch_object($result))
      $pagedPicBag[] = $row;

    $result = db_query('SELECT path, folder, gid FROM all_flat');
    $unpagedPicBag = array ();
    while ($row = db_fetch_object($result))
      $unpagedPicBag[] = $row;

    return array (
      'paged' => $pagedPicBag,
      'unpaged' => $unpagedPicBag
    );
  }

  /**
   * Get all images in given directory.
   *
   * @param path
   *   Absolute path of the directory
   * @param recursive
   *   Specify whether to recurse through subdirectories.
   * @return
   *   Array of arrays of image files. Each array element corresponds to
   *   an image type. If recursive is on, returns an array tree hierarchy.
   */
  public function exploreDir($path = '', $recursive = FALSE) {
    // List of image extensions
    $exts = array (
      'jpg',
      'jpeg',
      'png',
      'gif',
      'bmp',
      'flv',
      'mov',
      'wmv',
      'asx',
      'swf',
      'pdf',
      
    );

    // Get all image files of each file type
    $files = array ();
    foreach ($exts as $ext) {
      $pattern = sql_regcase($path . '*.' . $ext);
      $files[] = glob($pattern);
    }

    // Recurse through subdirectories if necessary
    if ($recursive) {
      $dirs = glob($path . '*', GLOB_MARK | GLOB_ONLYDIR);
      if (is_array($dirs)) {
        foreach ($dirs as $dir) {
          $files[] = $this->exploreDir($dir, TRUE);
        }
      }
    }

    return $files;
  }

  /**
   * Checks given extension against supported video types.
   *
   * @param ext
   *   Extension to check.
   */
  public function isVideoExt($ext) {
    $types = array (
      'flv',
      'mov',
      'wmv',
      'asx',
      'swf',
      
    );
    return in_array($ext, $types);
  }

  /*
   * Returns framework index given a file path. Index 4 and higher are
   * non-image file types (i.e. movie files, PDF).
   *
   * @param path
   *   Path to the file.
   */
  public function getFramework($path) {
    $path_ext_ar = explode('.', $path);
    $path_ext = strtolower(end($path_ext_ar));

    $framework = variable_get('fast_gallery_js_framework', 0);

    if ($this->isVideoExt($path_ext))
      $framework = 4;
    else
      if ($path_ext == 'pdf')
        $framework = 5;

    return $framework;
  }
}